@@ -1,3 +1,5 @@
+# LICENSE: AGPLv3. See LICENSE at root of repo
+
 import argparse
 import json
 import os
@@ -20,6 +22,7 @@ class GameDefinition:
         self.display_name = display_name
         self.install_folder = install_folder
         self.launch_arguments = launch_arguments
+        self.uri = f"com.epicgames.launcher://apps/{app_name}?action=launch&silent=true"
 
 
 class SteamAccount:
@@ -76,6 +79,14 @@ def parse_arguments():
         required=False,
     )
 
+    parser.add_argument(
+        "--use-paths",
+        default=False,
+        action="store_true",
+        help="Use the path to the executable (eg `C:\\Fortnite\\Fortnite.exe`) instead of an Epic Games"
+        + "Launcher URI (`com.epicgames.launcher://apps/fortnite?action=launch&silent=true`).",
+        required=False,
+    )
     return parser.parse_args()
 
 
@@ -89,7 +100,8 @@ def egs_collect_games(egs_manifest_path):
     games = list()
 
     for path in pathlist:
-        with open(path, "r") as f:
+        # EGS seems to write their json files out as utf-8
+        with open(path, "r", encoding="utf-8") as f:
             item = json.load(f)
 
             app_name = path
@@ -101,27 +113,27 @@ def egs_collect_games(egs_manifest_path):
                 display_name = item["DisplayName"]
 
             if item["bIsIncompleteInstall"]:
-                print(f"\tSkipping '{display_name}' since installation is incomplete")
+                print(f"\t- Skipping '{display_name}' since installation is incomplete")
                 continue
             elif not item["bIsApplication"]:
-                print(f"\tSkipping '{display_name}' since it isn't an application")
+                print(f"\t- Skipping '{display_name}' since it isn't an application")
                 continue
             elif "games" not in item["AppCategories"]:
-                print(f"\tSkipping '{display_name}' since it doesn't have the category 'games'")
+                print(f"\t- Skipping '{display_name}' since it doesn't have the category 'games'")
                 continue
 
             if "InstallLocation" not in item:
-                print(f"\tSkipping '{display_name}' since it apparently doesn't have an 'InstallLocation'")
+                print(f"\t- Skipping '{display_name}' since it apparently doesn't have an 'InstallLocation'")
                 continue
 
             install_location = os.path.normpath(item["InstallLocation"])
 
             if "LaunchExecutable" not in item:
-                print(f"\tSkipping '{display_name}' since it apparently doesn't have an executable")
+                print(f"\t- Skipping '{display_name}' since it apparently doesn't have an executable")
                 continue
 
             if "LaunchCommand" not in item:
-                print(f"'{display_name}' doesn't have LaunchCommands?")
+                print(f"\t- '{display_name}' doesn't have LaunchCommands?")
                 launch_arguments = ""
             else:
                 # I think this is for command line arguments...?
@@ -133,16 +145,22 @@ def egs_collect_games(egs_manifest_path):
             # (D:\Epic Games\RiME + /RiME/SirenGame/Binaries/Win64/RiME.exe = D:/RiME/SirenGame/Binaries/Win64/RiME.exe ???)
             # complete_path = os.path.join(install_location, launch_executable)
 
-            complete_path = (
+            executable_path = (
                 os.path.normpath(install_location) + os.path.sep + os.path.normpath(launch_executable)
             )
 
-            if not os.path.exists(complete_path):
-                print(install_location, launch_executable)
-                print(f"\tWarning: path `{complete_path}` does not exist for game {display_name}, excluding!")
+            # found by looking creating a shortcut on the desktop in the EGL and inspecting it
+            # using the URI instead of executable_path allows some games with online services
+            # to work (eg GTAV)
+
+            if not os.path.exists(executable_path):
+                print(
+                    f"\t- Warning: path `{executable_path}` does not exist for game {display_name}, skipping!"
+                )
                 continue
+
             games.append(
-                GameDefinition(complete_path, display_name, app_name, install_location, launch_arguments,)
+                GameDefinition(executable_path, display_name, app_name, install_location, launch_arguments)
             )
 
     print(f"Collected {len(games)} games from the EGS manifest store")
@@ -195,7 +213,11 @@ def enumerate_steam_accounts(steam_path):
             # we need to look inside this user's localconfig.vdf to figure out their
             # display name
 
-            with open(os.path.join(child.path, "config/localconfig.vdf"), "r") as localconfig:
+            # here we just replace any malformed characters since we are only doing this to get the
+            # display name
+            with open(
+                os.path.join(child.path, "config/localconfig.vdf"), "r", encoding="utf-8", errors="replace"
+            ) as localconfig:
                 cfg = vdf.load(localconfig)
                 username = cfg["UserLocalConfigStore"]["friends"]["PersonaName"]
                 accounts.append(SteamAccount(steamid, username))
@@ -211,7 +233,7 @@ def prompt_for_steam_account(accounts):
     Returns a steamid
     """
     if len(accounts) == 1:
-        return accounts[0][0]
+        return accounts[0].steamid
 
     print("Found multiple Steam accounts:")
     row_fmt = "{: >3} | {: <17} | {: <25}"
@@ -234,16 +256,22 @@ def prompt_for_steam_account(accounts):
         exit(-1)
 
 
-def to_shortcut(game):
+def to_shortcut(game, use_executable_path):
     """
     Turns the given GameDefinition into a shortcut dict, suitable to injecting
     into Steam's shortcuts.vdf
     """
+
+    if use_executable_path:
+        shortcut = game.executable_path
+    else:
+        shortcut = game.uri
+
     return {
         "appname": game.display_name,
-        "Exe": game.executable_path,
+        "Exe": shortcut,
         "StartDir": game.install_folder,
-        "icon": "",
+        "icon": game.executable_path,
         "ShortcutPath": "",
         "LaunchOptions": game.launch_arguments,
         "IsHidden": 0,
@@ -257,11 +285,19 @@ def to_shortcut(game):
     }
 
 
-def add_games_to_shortcut_file(steam_path, steamid, games, skip_backup):
+def add_games_to_shortcut_file(steam_path, steamid, games, skip_backup, use_executable_path):
+    if use_executable_path:
+        print()
+        print("⚠ ⚠ WARNING: ⚠ ⚠")
+        print("Using the path to the executable instead of the Epic Games Launcher URI")
+        print("You may experience issues with online games (eg GTAV!)")
+        print()
+
     shortcut_file_path = os.path.join(steam_path, "userdata", steamid, "config", "shortcuts.vdf")
 
     if not os.path.exists(shortcut_file_path):
         print(f"Could not find shortcuts file at `{shortcut_file_path}`")
+        print("Make a shortcut in Steam (Library ➡ ➕ Add Game ➡ Add a Non-Steam Game...) first. Aborting.")
         exit(-2)
 
     # read in the shortcuts file
@@ -280,11 +316,12 @@ def add_games_to_shortcut_file(steam_path, steamid, games, skip_backup):
     added = 0
     last_index = int(max(shortcuts["shortcuts"].keys()))
     for game in games:
-        if game.executable_path in all_paths:
+        shortcut = game.executable_path if use_executable_path else game.uri
+        if shortcut in all_paths:
             print(f"Not creating shortcut for `{game.display_name}` since it already has one")
             continue
         last_index += 1
-        shortcuts["shortcuts"][str(last_index)] = to_shortcut(game)
+        shortcuts["shortcuts"][str(last_index)] = to_shortcut(game, use_executable_path)
         added += 1
 
     print(f"Added {added} new games")
@@ -307,6 +344,8 @@ def add_games_to_shortcut_file(steam_path, steamid, games, skip_backup):
         shortcut_file.write(new_bytes)
 
     print("Updated `shortcuts.vdf` successfully!")
+    print()
+    print("➡   Restart Steam!")
 
 
 ####################################################################################################
@@ -326,6 +365,11 @@ if __name__ == "__main__":
     steamid = args.steamid
     accounts = enumerate_steam_accounts(args.steam_path)
 
+    if len(accounts) == 1 and steamid != "":
+        print(
+            "FYI: There is only one Steam account found on your computer, so you don't need to provide --steamid"
+        )
+
     # if they gave a username, find the steamid associated with it
     if not steamid.isdigit() and steamid != "":
         username = steamid
@@ -334,11 +378,18 @@ if __name__ == "__main__":
                 steamid = account.steamid
         if username == steamid:
             # bit hackish, triggers selection below if the username wasn't found
-            print(f"!!! SteamID for `{steamid}` not found.")
+            print("⚠ ⚠ WARNING: ⚠ ⚠")
+            print(f"SteamID for `{steamid}` not found!")
+            print("The SteamID you provided could not be found on your local machine.")
+            print(
+                "If you are providing the human readable name and not a numeric SteamID, make sure you spell it correctly"
+            )
+
             steamid = ""
 
     if steamid == "":
         steamid = prompt_for_steam_account(accounts)
 
     print(f"Installing shortcuts for SteamID `{steamid}`")
-    add_games_to_shortcut_file(args.steam_path, steamid, games, args.live_dangerously)
+    add_games_to_shortcut_file(args.steam_path, steamid, games, args.live_dangerously, args.use_paths)
+    print("Done.")
